#!/bin/bash
#
# run.sh
# NOTE: assumes commands dirname and egrep are present
#
# Usage:
#  ./run.sh <service-name>
#

# Bail out if we're being passed an incorrect number of arguments
if [ $# != 1 ] ; then
	exit 1
fi

echo -----  >> /tmp/mmUpdater.log
echo `date` >> /tmp/mmUpdater.log
echo Starting update for $1 >> /tmp/mmUpdater.log

daemon=/etc/init.d/$1

isSystemd=0

if [ ! -f $daemon ] ; then
    systemctl cat $1 > /dev/null 2>&1

    if [ $? -eq 0 ] ; then
        echo systemd installation detected >> /tmp/mmUpdater.log
        isSystemd=1
    else
	    echo error: unable to find $daemon or systemd configuration files >> /tmp/mmUpdater.log
	    exit 1
    fi
fi

# Move into folder containing this script
cd `dirname "$0"`

name=
dest=
if [ $isSystemd == 0 ] ; then
    # INIT script
    # Determine binary name
    name=`egrep "^(PROGRAM=)" $daemon`
    if [ -z $name ] ; then
        # Ubuntu has NAME instead of PROGRAM
        name=`egrep "^(NAME=)" $daemon`
        if [ -z $name ] ; then
            exit 1
        fi

        name=${name:5} # Strip "NAME=" from the name
    else
        name=${name:8} # Strip "PROGRAM=" from the name
    fi

    # Determine binary location
    dest=`egrep "^(LOCATION=)" $daemon`
    if [ -z $dest ] ; then
        exit 1
    fi

    dest=${dest:9} # Strip "LOCATION=" from the destination

    if [ ${dest:0:1} == "\"" -a ${dest:(-1)} == "\"" ] ; then
            dest=${dest:1:${#dest}-2} # Strip quotes from the destination
    fi
else
    # Systemd
    name=$1d
    dest=/usr/sbin
fi

# Replace binary
echo copying the new binary >> /tmp/mmUpdater.log
mv $dest/$name $dest/$name.bak  >> /tmp/mmUpdater.log
cp $1d $dest/$name  >> /tmp/mmUpdater.log

# Replace restartdhcp.sh script if present
if [ $isSystemd == 0 ] && [ -f ./restartdhcp.sh ] ; then
		paramFile="/etc/init.d/mmdhcpremote"
		if [ -f /etc/default/mmdhcpremote ] ; then
				paramFile="/etc/default/mmdhcpremote"
		fi
		dataDir=`egrep "(^DATA=)" $paramFile | cut -d= -f2`
		if [[ -d $dataDir && $dataDir != "" ]] ; then
			echo Copying restartdhcp.sh to $dataDir/scripts/ >> /tmp/mmUpdater.log
			cp restartdhcp.sh $dataDir/scripts/restartdhcp.sh  >> /tmp/mmUpdater.log
		fi
fi

# If there exist a new version of rest-doc in this package then we will 
# copy it to the correct place
# NOTE: We assume that /var/mmsuite/web_services exist, which should be the case if mmws has ever been started
if [ -d ./rest-doc ] ; then
    echo "Updating REST documentation for web services " >> /tmp/mmUpdater.log
    rm -rf /var/mmsuite/web_services/doc
    cp -r ./rest-doc /var/mmsuite/web_services/doc
fi

# Start daemon
echo Restarting $1 >> /tmp/mmUpdater.log
if [ $isSystemd == 0 ] ; then
    $daemon restart  >> /tmp/mmUpdater.log
else
    systemctl restart $1 >> /tmp/mmUpdater.log
fi

# If this is the web app upgrader, run the script to upgrade it.  Note that we do this last of all because it could fail 
# on systems where only mmws is installed.  If that happens, we want the new mmws to have been started so that it will 
# report the correct version the next time it is called
if [ -d ./webapp ] ; then
    echo "Attempting web app update " >> /tmp/mmUpdater.log
    if [ -f /var/mmsuite/web_services/upgrade_web_app.sh ] ; then
        /var/mmsuite/web_services/upgrade_web_app.sh ./webapp >> /tmp/mmUpdater.log 2>&1
        echo "web app update succesful" >> /tmp/mmUpdater.log
    else
        echo "warning: Unable to find upgrade script at /var/mmsuite/web_services/upgrade_web_app.sh" >> /tmp/mmUpdater.log
    fi
fi

echo Done updating  >> /tmp/mmUpdater.log

exit 0

